@@ -103,9 +103,9 @@
          else if (.lgrep("\\.rds$",obj)) {
             g1 <- session_grid()
             coast_xy <- readRDS(obj)
-            if (nchar(g1$proj)) {
+            if (nchar(g1$crs)) {
               # b <- attributes(coast_xy)
-               coast_xy <- .project(coast_xy,g1$proj4)
+               coast_xy <- .project(coast_xy,g1$crs)
               # attributes(coast_xy) <- b
             }
          }
@@ -124,11 +124,13 @@
          return(res)
       }
    }
-   isLongLat <- .lgrep("(\\+proj=longlat|epsg:4326)",g1$proj4)>0
-   isMerc <- .lgrep("\\+proj=merc",g1$proj4)>0
-   isCea <- .lgrep("\\+proj=cea",g1$proj4)>0
-   isUTM <- .lgrep("\\+proj=utm",g1$proj4)>0
-   proj <- g1$proj4
+   isLongLat <- .lgrep("(\\+proj=longlat|epsg:4326)",g1$crs)>0
+   isMerc <- .lgrep("\\+proj=merc",g1$crs)>0
+   isCea <- .lgrep("\\+proj=cea",g1$crs)>0
+   isUTM <- .lgrep("\\+proj=utm",g1$crs)>0
+   proj <- g1$crs
+   if ((is.list(proj))&&("input" %in% names(proj)))
+      proj <- proj[["input"]]
    proj <- proj[nchar(proj)==max(nchar(proj))]
    if ((any(is.na(proj)))||(nchar(proj)==0))
       return(NULL)
@@ -147,7 +149,7 @@
    {
       g2 <- with(g1,my.expand.grid(x=seq(minx,maxx,length=16)
                                   ,y=seq(miny,maxy,length=16)))
-      ll <- .project(with(g2,cbind(x,y)),g1$proj4,inv=TRUE)
+      ll <- .project(with(g2,cbind(x,y)),g1$crs,inv=TRUE)
       if (all(ll[,2]<65))
          return(NULL)
    }
@@ -184,7 +186,7 @@
       }
       else if (!isLoaded) {
          if (!requireNamespace("proj4",quietly=.isPackageInUse()))
-            requireNamespace("rgdal")
+            requireNamespace("rgdal",quietly=.isPackageInUse())
       }
    }
    else
@@ -648,13 +650,14 @@
    if (verbose)
       n <- nrow(a)
    aG <- sf::st_geometry(a)
-   sf::st_geometry(a) <- NULL
-   a <- sf::st_sf(data.frame(FID=as.integer(a$FID)),geom=aG) ## set my name for geometry: 'geom'
+  # sf::st_geometry(a) <- NULL
+  # a <- sf::st_sf(data.frame(FID=as.integer(a$FID)),geometry=aG)
+   spatial_data(a) <- data.frame(FID=as.integer(a$FID))
    .elapsedTime("reading - done")
    FID <- a$FID
    prj1 <- spatial_crs(a)
    lon0 <- 120
-   prj1 <- spatial_proj4(a)
+  # prj1 <- spatial_proj4(a) ## commented this 'x'-line after 'x-2' line assign 
    prj2 <- .gsub("(^.+)(\\s\\+lon_0=)(\\d+(\\.\\d+)*)(\\s.+$)"
                 ,paste0("\\1\\2",lon0,"\\5"),prj1)
    if (merge) {
@@ -675,13 +678,16 @@
          close(pb)
       a180 <- subset(a,FID %in% cross180)
       sf::st_agr(a180) <- "constant"
+     # spatial_write(a180,"a180.sqlite")
       c180 <- sf::st_transform(sf::st_centroid(a180),crs=4326)
       g180 <- do.call("rbind",lapply(lapply(sf::st_geometry(c180),unclass)
                                     ,matrix,ncol=2))
       sf::st_geometry(c180) <- NULL
-      c180 <- data.frame(c180,lon=g180[,1],lat=g180[,2])
-      if (verbose)
+      c180 <- data.frame(c180,lon=g180[,1],lat=g180[,2],area=spatial_area(a180)*1e-6)
+      if (verbose) {
          print(c180)
+         print(nrow(c180)) 
+      }
       p1 <- c180$FID[c180$lat>52.0 & c180$lat<67.0] # Chukotka
       p2 <- c180$FID[c180$lat>71.0 & c180$lat<71.5] # Wrangel
       p3 <- c180$FID[c180$lat>(-17.0) & c180$lat<(-16.7)] # Taveuni (south)
@@ -689,6 +695,7 @@
       p5 <- c180$FID[c180$lat>(-16.65) & c180$lat<(-16.55) |
                      c180$lat>(-16.3) & c180$lat<(-16.0)] # Labasa (top MANUAL)
       pair <- list(Chukotka=p1,Wrangel=p2,Taveuni_S=p3,Rabi_M=p4,Labasa_N=p5)
+      pname <- names(pair)
       if (verbose) {
          print(pair)
          if (FALSE) {
@@ -697,36 +704,87 @@
             spatial_write(sf::st_transform(a180,prj2),"cross180.sqlite")
          }
       }
+      if ((!.isPackageInUse())&&(devel <- F)) {
+         a5 <- subset(a,FID %in% pair[[1]])
+         message("writing `wrangel_fail.sqlite`")
+         spatial_write(spatial_transform(a5,3571),"wrangel_fail.sqlite")
+         q()
+         ##~ a5 <- spatial_union(a5)
+         ##~ str(a5)
+         ##~ glance(a5,resetProj=TRUE)
+         ##~ q()
+      }
       a180 <- vector("list",length(pair))
       for (i in seq_along(pair)) {
          fid5 <- pair[[i]]
-         a5 <- subset(a,FID %in% fid5)
-         c5 <- subset(c180,FID %in% fid5)
-         a5 <- sf::st_transform(a5,crs=prj2)
-         if (i==5) {
-            ind <- c5$lon<0
-            closest <- fid5[which(ind)]
-            expanded <- fid5[which(!ind)]
-            pt <- sf::st_sfc(sf::st_point(c(179.99993,-16.16841)),crs=4326)
-            pt <- unclass(unclass(sf::st_transform(pt,crs=prj2))[[1]])
-            a5b <- subset(a5,FID==closest)
-            g5b <- unclass(unclass(sf::st_geometry(a5b))[[1]])[[1]]
-            db <- (abs(g5b[,1])-pt[1])^2+(g5b[,2]-pt[2])^2
-            indb <- which.min(db)
-            pt <- g5b[indb,]
-            a5a <- subset(a5,FID==expanded)
-            g5a <- unclass(unclass(sf::st_geometry(a5a))[[1]])[[1]]
-            da <- (abs(g5a[,1])-pt[1])^2+(g5a[,2]-pt[2])^2
-            inda <- which.min(da)
-            if (FALSE)
-               g5a <- rbind(g5a[1:inda,],pt,g5a[(inda+1):nrow(g5a),])
-            else
-               g5a <- rbind(g5a[1:(inda-1),],pt,g5a[inda:nrow(g5a),])
-            sf::st_geometry(a5a) <- sf::st_sfc(sf::st_polygon(list(g5a)),crs=prj2)
-            a5 <- rbind(a5a,a5b)
+         a0 <- spatial_transform(subset(a,FID %in% fid5),3571)
+         xy0 <- spatial_coordinates(a0)
+         xy1 <- head(xy0[[1]][[1]],-1)
+         xy2 <- head(xy0[[2]][[1]],-1)
+         d1 <- .dist2(xy1,xy2,verbose=FALSE)
+         d1$x <- xy1[,1]
+         order1 <- order(d1$dist,d1$x)[1:2]
+         ind1 <- d1$ind[order1]
+         d2 <- .dist2(xy2,xy1,verbose=FALSE)
+         d2$x <- xy2[,1]
+         order2 <- order(d2$dist,d2$x)[1:2]
+         ind2 <- d2$ind[order2]
+         if (ind2[1]>ind2[2]) { ## 10 9
+            xy7 <- rbind(xy1[ind2[1]:nrow(xy1),]
+                        ,xy1[1L:ind2[2],]
+                        )
          }
-         a5 <- sf::st_sf(FID=65000L+i,geom=sf::st_union(a5))
-         a180[[i]] <- sf::st_transform(a5,prj1)
+         else { ## 9 10
+            if (ind2[1]>1) {
+               xy7 <- rbind(xy1[ind2[1]:1L,]
+                           ,xy1[nrow(xy1):ind2[2],]
+                           )
+            }
+            else { ## 1 11
+               xy7 <- rbind(xy1[ind2[1]:ind2[2],])
+            }
+         }
+         if (ind1[1]>ind1[2]) { ## 6 1
+            if (ind1[2]>1) {
+               xy7 <- rbind(xy7
+                           ,xy2[ind1[2]:1L,]
+                           ,xy2[nrow(xy2):ind1[2],]
+                           )
+            }
+            else { ## 6 1
+               xy7 <- rbind(xy7
+                           ,xy2[ind1[2]:ind1[1],]
+                           )
+            }
+         }
+         else { ##
+            if (ind1[1]>1) {
+               if (.isPackageInUse())
+                  print("B15")
+               xy7 <- rbind(xy7
+                           ,xy2[ind1[2]:nrow(xy2),]
+                           ,xy2[1L:ind1[2],]
+                           )
+            }
+            else { ## 1 11
+               xy7 <- rbind(xy7
+                          # ,xy2[ind1[2]:ind1[1],]
+                           )
+            }
+         }
+         xy7 <- xy7[c(which(!duplicated(xy7))),]
+         xy7 <- xy7[c(seq(nrow(xy7)),1),]
+         if (mean(xy1[,1])<0) {
+            xy0[[1]][[1]] <- xy7
+            xy0[2] <- NULL
+         }
+         else {
+            xy0[[2]][[1]] <- xy7
+            xy0[1] <- NULL
+         }
+         a7 <- sf::st_sfc(GOMER=sf::st_polygon(xy0[[1]]),crs=spatial_crs(a0))
+         spatial_data(a7) <- data.frame(FID=65000L+i)
+         a180[[i]] <- sf::st_transform(a7,prj1)
       }
       .elapsedTime("spliting - done")
       a <- list(subset(a,!(FID %in% unlist(pair))))
